package cn.com.qmth.stmms.biz.report.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import cn.com.qmth.stmms.common.annotation.ExcelField;

/**
 * 课程机组成绩分析
 *
 * @author xuwenjin
 * @date 2019-04-12
 */
@Entity
@Table(name = "s_basic_subject_class", indexes = { @Index(columnList = "exam_id,subject_code,class_name") })
public class ReportSubjectClass implements Serializable {

    private static final long serialVersionUID = 1932022687226092788L;

    /**
     * 主键
     */
    @Id
    @GeneratedValue
    private Integer id;

    /**
     * 考试id
     */
    @Column(name = "exam_id")
    private Integer examId;

    /**
     * 课程编号
     */
    @ExcelField(title = "课程代码", align = 2, sort = 10)
    @Column(name = "subject_code")
    private String subjectCode;

    /**
     * 课程名称
     */
    @ExcelField(title = "课程名称", align = 2, sort = 20)
    @Column(name = "subject_name")
    private String subjectName;

    /**
     * 机组
     */
    @ExcelField(title = "机组", align = 2, sort = 30)
    @Column(name = "class_name")
    private String className;

    /**
     * 最高分
     */
    @ExcelField(title = "最高分", align = 2, sort = 40)
    @Column(name = "max_score")
    private Double maxScore;

    /**
     * 最低分
     */
    @ExcelField(title = "最低分", align = 2, sort = 50)
    @Column(name = "min_score")
    private Double minScore;

    /**
     * 及格人数
     */
    @ExcelField(title = "及格人数", align = 2, sort = 60)
    @Column(name = "pass_count")
    private int passCount;

    /**
     * 及格率
     */
    @ExcelField(title = "及格率", align = 2, sort = 70)
    @Column(name = "pass_rate")
    private Double passRate;

    /**
     * 优秀数
     */
    @ExcelField(title = "优秀人数", align = 2, sort = 80)
    @Column(name = "excellent_count")
    private int excellentCount;

    /**
     * 优秀率
     */
    @ExcelField(title = "优秀率", align = 2, sort = 90)
    @Column(name = "excellent_rate")
    private Double excellentRate;

    /**
     * 平均分
     */
    @ExcelField(title = "平均分", align = 2, sort = 100)
    @Column(name = "avg_score")
    private Double avgScore;

    /**
     * 报考人数
     */
    @ExcelField(title = "报考人数", align = 2, sort = 110)
    @Column(name = "total_count")
    private Integer totalCount;

    /**
     * 有效人数
     */
    @ExcelField(title = "有效人数", align = 2, sort = 120)
    @Column(name = "reality_count")
    private Integer realityCount;

    /**
     * 差异系数
     */
    @Column(name = "coefficient", nullable = true, length = 32)
    private Double coefficient;

    /**
     * 难度
     */
    @Column(name = "difficulty", nullable = true, length = 32)
    private Double difficulty;

    /**
     * 标准差
     */
    @Column(name = "stdev")
    private Double stdev;

    /**
     * 高低分段统计
     */
    @Column(name = "range_level", nullable = true)
    private String rangeLevel;

    /**
     * 难度分布
     */
    @Column(name = "difficulity_level", nullable = true)
    private String difficulityLevel;

    /**
     * 区分度分布
     */
    @Column(name = "discrimination_level", nullable = true)
    private String discriminationLevel;

    /**
     * 一分一段分段统计
     */
    @Column(name = "score_range", nullable = true, columnDefinition = "text")
    private String scoreRange;

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Integer getExamId() {
        return examId;
    }

    public void setExamId(Integer examId) {
        this.examId = examId;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public void setSubjectCode(String subjectCode) {
        this.subjectCode = subjectCode;
    }

    public String getSubjectName() {
        return subjectName;
    }

    public void setSubjectName(String subjectName) {
        this.subjectName = subjectName;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public Double getMaxScore() {
        return maxScore;
    }

    public void setMaxScore(Double maxScore) {
        this.maxScore = maxScore;
    }

    public Double getMinScore() {
        return minScore;
    }

    public void setMinScore(Double minScore) {
        this.minScore = minScore;
    }

    public int getPassCount() {
        return passCount;
    }

    public void setPassCount(int passCount) {
        this.passCount = passCount;
    }

    public Double getPassRate() {
        return passRate;
    }

    public void setPassRate(Double passRate) {
        this.passRate = passRate;
    }

    public int getExcellentCount() {
        return excellentCount;
    }

    public void setExcellentCount(int excellentCount) {
        this.excellentCount = excellentCount;
    }

    public Double getExcellentRate() {
        return excellentRate;
    }

    public void setExcellentRate(Double excellentRate) {
        this.excellentRate = excellentRate;
    }

    public Double getAvgScore() {
        return avgScore;
    }

    public void setAvgScore(Double avgScore) {
        this.avgScore = avgScore;
    }

    public Double getCoefficient() {
        return coefficient;
    }

    public void setCoefficient(Double coefficient) {
        this.coefficient = coefficient;
    }

    public Double getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(Double difficulty) {
        this.difficulty = difficulty;
    }

    public Double getStdev() {
        return stdev;
    }

    public void setStdev(Double stdev) {
        this.stdev = stdev;
    }

    public String getRangeLevel() {
        return rangeLevel;
    }

    public void setRangeLevel(String rangeLevel) {
        this.rangeLevel = rangeLevel;
    }

    public String getDifficulityLevel() {
        return difficulityLevel;
    }

    public void setDifficulityLevel(String difficulityLevel) {
        this.difficulityLevel = difficulityLevel;
    }

    public String getDiscriminationLevel() {
        return discriminationLevel;
    }

    public void setDiscriminationLevel(String discriminationLevel) {
        this.discriminationLevel = discriminationLevel;
    }

    public String getScoreRange() {
        return scoreRange;
    }

    public void setScoreRange(String scoreRange) {
        this.scoreRange = scoreRange;
    }

    public Integer getRealityCount() {
        return realityCount;
    }

    public void setRealityCount(Integer realityCount) {
        this.realityCount = realityCount;
    }

    public Integer getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(Integer totalCount) {
        this.totalCount = totalCount;
    }

}